<?php

/**
 * Shopgate GmbH
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file AFL_license.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to interfaces@shopgate.com so we can send you a copy immediately.
 *
 * @author    Shopgate GmbH, Schloßstraße 10, 35510 Butzbach <interfaces@shopgate.com>
 * @copyright Shopgate GmbH
 * @license   http://opensource.org/licenses/AFL-3.0 Academic Free License ("AFL"), in the version 3.0
 */
class ShopgateCouponHelper
{

    /**
     * @param ShopgateCartBase $cartBase
     * @param int              $customerId
     * @param int              $langId
     * @param int              $currencyId
     *
     * @return CartRule[]
     */
    public function createVirtualCartRules(ShopgateCartBase $cartBase, $customerId, $langId, $currencyId)
    {
        $cartRules = array();

        if (_PS_VERSION_ < '1.5') {
            return $cartRules;
        }

        foreach ($cartBase->getItems() as $cartItem) {
            /** @var $item ShopgateOrderItem */
            if ($cartItem->getType() !== ShopgateOrderItem::TYPE_SHOPGATE_COUPON) {
                continue;
            }

            $cartRule = $this->createCartRule(
                $customerId,
                $langId,
                $cartItem->getName(),
                $cartItem->getUnitAmount(),
                $currencyId
            );

            if (!Validate::isLoadedObject($cartRule)) {
                continue;
            }

            $cartRules[] = $cartRule;
        }

        return $cartRules;
    }

    /**
     * @param Cart       $prestashopCart
     * @param CartRule[] $cartRules
     */
    public function applyCartRules($prestashopCart, array $cartRules)
    {
        if (_PS_VERSION_ < '1.5') {
            return;
        }

        foreach ($cartRules as $cartRule) {
            $prestashopCart->addCartRule($cartRule->id);
        }
    }

    /**
     * @param int    $customerId
     * @param int    $langId
     * @param string $cartRuleName
     * @param float  $cartRuleAmount
     * @param int    $currencyId
     *
     * @return CartRule
     */
    protected function createCartRule($customerId, $langId, $cartRuleName, $cartRuleAmount, $currencyId)
    {
        $cartRule                     = new CartRule();
        $cartRule->name               = array($langId => $cartRuleName);
        $cartRule->reduction_amount   = abs($cartRuleAmount);
        $cartRule->id_customer        = $customerId;
        $cartRule->date_from          = date('Y-m-d');
        $cartRule->date_to            = date('Y-m-d 23:59:59');
        $cartRule->reduction_currency = $currencyId;
        $cartRule->add();

        return $cartRule;
    }
}
