<?php

/**
 * Shopgate GmbH
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file AFL_license.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to interfaces@shopgate.com so we can send you a copy immediately.
 *
 * @author    Shopgate GmbH, Schloßstraße 10, 35510 Butzbach <interfaces@shopgate.com>
 * @copyright Shopgate GmbH
 * @license   http://opensource.org/licenses/AFL-3.0 Academic Free License ("AFL"), in the version 3.0
 */
class ShopgateItemAttributeHelper
{
    /** @var int */
    private $exportLanguageId;

    /**
     * @param int $exportLanguageId
     */
    public function __construct($exportLanguageId)
    {
        $this->exportLanguageId = $exportLanguageId;
    }

    /**
     * We use method "getAttributeCombinaisons" (instead of method "getAttributeCombinations")
     * because its available in all Prestashop versions. In later versions method "getAttributeCombinaisons" will
     * just call method "getAttributeCombinations"
     *
     * @param Product $prestashopProduct
     *
     * @return ShopgateItemsProductAttribute[]
     */
    public function getProductAttributes(Product $prestashopProduct)
    {
        $prestashopAttributes = $prestashopProduct->getAttributeCombinaisons($this->exportLanguageId);

        $productAttributes = array();
        foreach ($prestashopAttributes as $prestashopAttribute) {
            if (!isset($productAttributes[$prestashopAttribute['id_product_attribute']])) {
                $productAttribute = new ShopgateItemsProductAttribute(
                    $prestashopAttribute['id_product_attribute'],
                    $prestashopAttribute['default_on'],
                    $prestashopAttribute['wholesale_price'],
                    $prestashopAttribute['minimal_quantity'],
                    $prestashopAttribute['unit_price_impact'],
                    $prestashopAttribute['quantity']
                );

                $productAttributes[$prestashopAttribute['id_product_attribute']] = $productAttribute;
            }

            $productAttribute = $productAttributes[$prestashopAttribute['id_product_attribute']];

            $productAttribute->addAttributeCombination(
                new ShopgateItemsAttributeCombination(
                    new ShopgateItemsAttributeGroup(
                        $prestashopAttribute['id_attribute_group'],
                        $prestashopAttribute['group_name']
                    ),
                    new ShopgateItemsAttribute(
                        $prestashopAttribute['id_attribute'],
                        $prestashopAttribute['attribute_name']
                    )
                )
            );
        }

        return $productAttributes;
    }

    /**
     * @param Product $prestashopProduct
     *
     * @return ShopgateItemsAttributeGroup[]
     */
    public function getAttributeGroups(Product $prestashopProduct)
    {
        $attributeGroups = array();
        foreach ($this->getProductAttributes($prestashopProduct) as $productAttribute) {
            foreach ($productAttribute->getAttributeCombinations() as $productCombination) {
                $attributeGroups[] = $productCombination->getAttributeGroup();
            }
            break;
        }

        return $attributeGroups;
    }
}