<?php
/**
 * Shopgate GmbH
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file AFL_license.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to interfaces@shopgate.com so we can send you a copy immediately.
 *
 * @author    Shopgate GmbH, Schloßstraße 10, 35510 Butzbach <interfaces@shopgate.com>
 * @copyright Shopgate GmbH
 * @license   http://opensource.org/licenses/AFL-3.0 Academic Free License ("AFL"), in the version 3.0
 */

class ShopgateItemsInput
{
    /**
     * @var ShopgateDb
     */
    protected $db;

    public function __construct(ShopgateDb $db)
    {
        $this->db = $db;
    }

    /**
     * Creates customizations and references them to the cart
     *
     * @param ShopgateOrderItem $item
     * @param string | int      $cartId
     * @param int               $idAddressDelivery
     *
     * @return bool
     */
    public function createCustomization(ShopgateOrderItem $item, $cartId, $idAddressDelivery = 0)
    {
        if (!$item->getInputs()) {
            return false;
        }

        $insert = array(
            'id_cart'    => $cartId,
            'id_product' => $item->getItemNumber(),
        );
        if (version_compare(_PS_VERSION_, '1.5.0.0', '>=')) {
            $insert['id_address_delivery'] = $idAddressDelivery;
            $insert['in_cart']             = 1;
        }
        $this->db->getInstance()->autoExecuteWithNullValues(_DB_PREFIX_ . 'customization', $insert, 'INSERT');

        return $this->db->getInstance()->Insert_ID();
    }

    /**
     * @param int                      $customizationId
     * @param ShopgateOrderItemInput[] $inputs
     */
    public function createCustomizedData($customizationId, $inputs)
    {
        if (empty($inputs) || empty($customizationId)) {
            return;
        }

        foreach ($inputs as $input) {
            $type = $this->getSystemInputType($input);
            $this->db->getInstance()->autoExecuteWithNullValues(
                _DB_PREFIX_ . 'customized_data',
                array(
                    'id_customization' => $customizationId,
                    'type'             => $type,
                    'index'            => $input->getInputNumber(),
                    'value'            => $input->getUserInput()
                ),
                'INSERT'
            );
        }
    }

    /**
     * Translates shopgate input type into
     * system type ID to save to the DB
     * 1 - text field
     * 2 - file field, we current do not support it though
     *
     * @param ShopgateOrderItemInput $input
     *
     * @return int
     */
    public function getSystemInputType(ShopgateOrderItemInput $input)
    {
        return $input->getType() === Shopgate_Model_Catalog_Input::DEFAULT_INPUT_TYPE_TEXT ? 1 : 2;
    }
}
