<?php

/**
 * Shopgate GmbH
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file AFL_license.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to interfaces@shopgate.com so we can send you a copy immediately.
 *
 * @author    Shopgate GmbH, Schloßstraße 10, 35510 Butzbach <interfaces@shopgate.com>
 * @copyright Shopgate GmbH
 * @license   http://opensource.org/licenses/AFL-3.0 Academic Free License ("AFL"), in the version 3.0
 */
class ShopgateItemsProductAttribute
{
    /** @var int */
    private $id;

    /** @var bool */
    private $defaultOn;

    /** @var float */
    private $wholeSalePrice;

    /** @var int */
    private $minimalQuantity;

    /** @var float */
    private $unitPriceImpact;

    /** @var int */
    private $quantity;

    /** @var ShopgateItemsAttributeCombination[] */
    private $attributeCombinations;

    /**
     * @param int   $id
     * @param bool  $defaultOn
     * @param float $wholeSalePrice
     * @param int   $minimalQuantity
     * @param float $unitPriceImpact
     * @param int   $quantity
     */
    public function __construct(
        $id,
        $defaultOn,
        $wholeSalePrice,
        $minimalQuantity,
        $unitPriceImpact,
        $quantity
    ) {
        $this->id                    = $id;
        $this->defaultOn             = $defaultOn;
        $this->wholeSalePrice        = $wholeSalePrice;
        $this->minimalQuantity       = $minimalQuantity;
        $this->unitPriceImpact       = $unitPriceImpact;
        $this->quantity              = $quantity;
        $this->attributeCombinations = array();
    }

    /**
     * @return int
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * @param int $id
     */
    public function setId($id)
    {
        $this->id = $id;
    }

    /**
     * @return bool
     */
    public function isDefaultOn()
    {
        return $this->defaultOn;
    }

    /**
     * @param bool $defaultOn
     */
    public function setDefaultOn($defaultOn)
    {
        $this->defaultOn = $defaultOn;
    }

    /**
     * @return float
     */
    public function getWholeSalePrice()
    {
        return $this->wholeSalePrice;
    }

    /**
     * @param float $wholeSalePrice
     */
    public function setWholeSalePrice($wholeSalePrice)
    {
        $this->wholeSalePrice = $wholeSalePrice;
    }

    /**
     * @return int
     */
    public function getMinimalQuantity()
    {
        return $this->minimalQuantity;
    }

    /**
     * @param int $minimalQuantity
     */
    public function setMinimalQuantity($minimalQuantity)
    {
        $this->minimalQuantity = $minimalQuantity;
    }

    /**
     * @return float
     */
    public function getUnitPriceImpact()
    {
        return $this->unitPriceImpact;
    }

    /**
     * @param float $unitPriceImpact
     */
    public function setUnitPriceImpact($unitPriceImpact)
    {
        $this->unitPriceImpact = $unitPriceImpact;
    }

    /**
     * @return int
     */
    public function getQuantity()
    {
        return $this->quantity;
    }

    /**
     * @param int $quantity
     */
    public function setQuantity($quantity)
    {
        $this->quantity = $quantity;
    }

    /**
     * @return ShopgateItemsAttributeCombination[]
     */
    public function getAttributeCombinations()
    {
        return $this->attributeCombinations;
    }

    /**
     * @param ShopgateItemsAttributeCombination $attributeCombination
     */
    public function addAttributeCombination(ShopgateItemsAttributeCombination $attributeCombination)
    {
        $this->attributeCombinations[] = $attributeCombination;
    }
}